@@ -140,18 +140,18 @@ gimp_display_shell_dnd_init (GimpDisplayShell *shell)
   gimp_dnd_color_dest_add     (shell->canvas,
                                gimp_display_shell_drop_color,
                                shell);
-  gimp_dnd_svg_dest_add       (shell->canvas,
-                               gimp_display_shell_drop_svg,
-                               shell);
   gimp_dnd_component_dest_add (shell->canvas,
                                gimp_display_shell_drop_component,
                                shell);
-  gimp_dnd_pixbuf_dest_add    (shell->canvas,
-                               gimp_display_shell_drop_pixbuf,
-                               shell);
   gimp_dnd_uri_list_dest_add  (shell->canvas,
                                gimp_display_shell_drop_uri_list,
                                shell);
+  gimp_dnd_svg_dest_add       (shell->canvas,
+                               gimp_display_shell_drop_svg,
+                               shell);
+  gimp_dnd_pixbuf_dest_add    (shell->canvas,
+                               gimp_display_shell_drop_pixbuf,
+                               shell);
 }
 
 
@@ -167,13 +167,13 @@ gimp_display_shell_dnd_position_item (GimpDisplayShell *shell,
 {
   gint item_width  = gimp_item_get_width  (item);
   gint item_height = gimp_item_get_height (item);
+  gint off_x, off_y;
 
   if (item_width  >= gimp_image_get_width  (image) &&
       item_height >= gimp_image_get_height (image))
     {
-      gimp_item_set_offset (item,
-                            (gimp_image_get_width  (image) - item_width)  / 2,
-                            (gimp_image_get_height (image) - item_height) / 2);
+      off_x = (gimp_image_get_width  (image) - item_width)  / 2;
+      off_y = (gimp_image_get_height (image) - item_height) / 2;
     }
   else
     {
@@ -182,10 +182,14 @@ gimp_display_shell_dnd_position_item (GimpDisplayShell *shell,
 
       gimp_display_shell_untransform_viewport (shell, &x, &y, &width, &height);
 
-      gimp_item_set_offset (item,
-                            x + (width  - item_width)  / 2,
-                            y + (height - item_height) / 2);
+      off_x = x + (width  - item_width)  / 2;
+      off_y = y + (height - item_height) / 2;
     }
+
+  gimp_item_translate (item,
+                       off_x - gimp_item_get_offset_x (item),
+                       off_y - gimp_item_get_offset_y (item),
+                       FALSE);
 }
 
 static void
