@@ -1303,12 +1303,15 @@ gimp_display_shell_reconnect (GimpDisplayShell *shell)
 void
 gimp_display_shell_empty (GimpDisplayShell *shell)
 {
-  GimpContext *user_context;
+  GimpContext     *user_context;
+  GimpImageWindow *window;
 
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (GIMP_IS_DISPLAY (shell->display));
   g_return_if_fail (gimp_display_get_image (shell->display) == NULL);
 
+  window = gimp_display_shell_get_window (shell);
+
   if (shell->fill_idle_id)
     {
       g_source_remove (shell->fill_idle_id);
@@ -1319,7 +1322,10 @@ gimp_display_shell_empty (GimpDisplayShell *shell)
 
   gimp_display_shell_unset_cursor (shell);
 
+  gimp_display_shell_sync_config (shell, shell->display->config);
+
   gimp_display_shell_appearance_update (shell);
+  gimp_image_window_update_tabs (window);
 #if 0
   gimp_help_set_help_data (shell->canvas,
                            _("Drop image files here to open them"), NULL);
@@ -1361,10 +1367,14 @@ gimp_display_shell_fill (GimpDisplayShell *shell,
                          GimpUnit          unit,
                          gdouble           scale)
 {
+  GimpImageWindow *window;
+
   g_return_if_fail (GIMP_IS_DISPLAY_SHELL (shell));
   g_return_if_fail (GIMP_IS_DISPLAY (shell->display));
   g_return_if_fail (GIMP_IS_IMAGE (image));
 
+  window = gimp_display_shell_get_window (shell);
+
   gimp_display_shell_set_unit (shell, unit);
   gimp_display_shell_set_initial_scale (shell, scale, NULL, NULL);
   gimp_display_shell_scale_changed (shell);
@@ -1372,6 +1382,7 @@ gimp_display_shell_fill (GimpDisplayShell *shell,
   gimp_display_shell_sync_config (shell, shell->display->config);
 
   gimp_display_shell_appearance_update (shell);
+  gimp_image_window_update_tabs (window);
 #if 0
   gimp_help_set_help_data (shell->canvas, NULL, NULL);
 #endif
